@@ -38,6 +38,9 @@ class Proxy():
     
     __amqpc = None
 
+    pull_commands_delay = 2
+    pull_commands_attempts = 42
+
     def __init__(self, actor:str, amqpc:BaseClient = None, **kwargs):
         """ init """
 
@@ -58,55 +61,71 @@ class Proxy():
         if not self.isAmqpcConnected():
             await self.amqpc.start()
 
-        await self.__pull_commands()
+        await self._pull_commands()
    
         return self
 
 
     async def stop(self):
         """stop actor"""
-        if hasattr(self, "__pull_commands_task"):
-            self.__pull_commands_task.cancel()
-            delattr(self ,"__pull_commands_task")
+        await self.__delattr_pull_commands_task(cancel=True)
 
     @staticmethod
     def setDefaultAmqpc(amqpc):
         Proxy.__amqpc = amqpc
 
+    #def __getattr__(self, attr):
+        #if attr != "_pull_commands_task":
+            #try:
+                #super(Proxy, self).__getattribute__(self, "ping")
 
-    def __getattr__(self, attr):
-        if attr != "__pull_commands_task":
+            #except AttributeError:
+                #self.amqpc.log.warning(f"actor {self.actor} {attr} currently not reachable.")
+                #setattr(self, attr, partial(self.call_command, attr))
+
+        #return super(Proxy, self).__getattribute__(attr)
+##        return self.__getattribute__(attr)
+
+    async def __delattr_pull_commands_task(self, cancel=False):
+        lock = asyncio.Lock()
+        async with lock:
+            if hasattr(self, "_pull_commands_task"):
+                if cancel:
+                    self._pull_commands_task.cancel()
+                    try:
+                        await self._pull_commands_task
+                    except asyncio.exceptions.CancelledError as ex:
+                        self.amqpc.log.debug(f"error {ex}")
+                delattr(self ,"_pull_commands_task")
+        
+
+    async def _pull_commands(self, delay = 0, attempts = 1):
+        for c in range(attempts):
             try:
-                object.__getattribute__(self, "ping")
+                await asyncio.sleep(delay)
 
-            except AttributeError:
-                self.amqpc.log.warning(f"actor {self.actor} {attr} currently not reachable.")
-                setattr(self, attr, partial(self.call_command, attr))
+                reply = await self.call_command(Proxy.__commands)
 
-        return self.__getattribute__(attr)
+                commands = reply[Proxy.__commands_key] if isinstance(reply, dict) else reply.help
 
-    async def __pull_commands(self, delay = 0):
+                for c in commands:
+                    setattr(self, c, partial(self.call_command, c))
+                    # setattr(self, f"nowait_{c}", partial(self.call_command, c, nowait=True))
 
-        try:
-            await asyncio.sleep(delay)
-
-            reply = await self.call_command(Proxy.__commands)
-
-            commands = reply[Proxy.__commands_key] if isinstance(reply, dict) else reply.help
-
-            for c in commands:
-                setattr(self, c, partial(self.call_command, c))
-                # setattr(self, f"nowait_{c}", partial(self.call_command, c, nowait=True))
-
-            if hasattr(self, "__pull_commands_task"):
-                delattr(self ,"__pull_commands_task")
+                await self.__delattr_pull_commands_task()
+                return
 
 
-        except Exception as ex:
-            if not delay:
-                self.amqpc.log.warning(f"actor {self.actor} currently not reachable.")
-            if not hasattr(self, "__pull_commands_task"):
-                self.__pull_commands_task = self.amqpc.loop.create_task(self.__pull_commands(2))
+            except Exception as ex:
+                if not delay:
+                    self.amqpc.log.warning(f"actor {self.actor} currently not reachable.")
+                if not hasattr(self, "_pull_commands_task"):
+                    self.amqpc.log.debug(f"attempt delayed connection as background task.")
+                    self._pull_commands_task = self.amqpc.loop.create_task(self._pull_commands(Proxy.pull_commands_delay, Proxy.pull_commands_attempts))
+                    return    
+
+        self.amqpc.log.debug(f"stop delayed connection as background task.")
+        await self.__delattr_pull_commands_task()
 
 
     def isAmqpcConnected(self):
